#ifndef CLICK_PSCACHE_HH
#define CLICK_PSCACHE_HH
#include <click/element.hh>
#include <click/atomic.hh>
#include "bintree.hh"
#include "pscheck.hh"

CLICK_DECLS

/*
=c

PSHash(OFFSET, SIZE)

=s pubsub

Calculates hash suitable for PSCache element. Sets the result in packet's annotations.

=d

Hash is calculated from the OFFSET of the beginning of the packet, and SIZE consequtive
bytes are used.

The hash result value is added to packet's annotations, otherwise packet is not modified.

*/

class PSCache : public Element { 
public:
    
    PSCache();
    ~PSCache();
    
    const char *class_name() const		{ return "PSCache"; }
    const char *port_count() const		{ return "0/0"; }
    const char *processing() const		{ return "a/a"; }
    const char *flow_code() const               { return "x/x"; }

    int configure(Vector<String> &, ErrorHandler *);
    void push(int, Packet *);

    void store(unsigned int hashvalue, unsigned char const* start, unsigned int len);
    Packet *load(unsigned int hashvalue, Packet *old);
    String print();

private:
    BinTree* _tree;
};


CLICK_ENDDECLS
#endif
