#ifndef CLICK_PSCLASSIFY_HH
#define CLICK_PSCLASSIFY_HH
#include <click/element.hh>
#include <click/atomic.hh>
#include "pscheck.hh"
#include "psclassify.hh"
CLICK_DECLS

class SimpleProg;
class SimpleIns;


/*
=c

PSHash(OFFSET, SIZE)

=s pubsub

Calculates hash suitable for PSCache element. Sets the result in packet's annotations.

=d

Hash is calculated from the OFFSET of the beginning of the packet, and SIZE consequtive
bytes are used.

The hash result value is added to packet's annotations, otherwise packet is not modified.

*/
class PSClassify : public Element { 
public:
    
    PSClassify();
    ~PSClassify();
    
    const char *class_name() const		{ return "PSClassify"; }
    const char *port_count() const		{ return "1/-"; }
    const char *processing() const		{ return "h/h"; }
    const char *flow_code() const               { return "x/xy"; }

    int configure(Vector<String> &, ErrorHandler *);
    void push(int, Packet *);
private:
    int create_program(Vector<String> &, ErrorHandler *);
    Vector<SimpleProg *> *sips;
};

class SimpleProg {
public:
    struct regs {
	unsigned int pc;
	int reg;
	bool negate;
	bool skip;
    };

    struct tokens {
	String token;
	int value;
	tokens(const char *str, int val) : token(str), value(val) {}
    };

    SimpleProg();
    ~SimpleProg();
    void add_cmd(int code, int output);
    bool terminate_cmd(int output);
    void skip_operand(struct SimpleProg::regs &mem);
    int run(Packet *p);

    void debug();
private:
    Vector<SimpleIns*> *ins;
};

class SimpleIns {
public:
    SimpleIns(int code, int output);
    ~SimpleIns();

    int execute(struct SimpleProg::regs &mem, Packet *p);
    int getCode();
private:
    int myout, mycode;
};




CLICK_ENDDECLS
#endif
