#include <click/config.h>
#include <click/confparse.hh>
#include "pshash.hh"
#include "psclassify.hh"

CLICK_DECLS

/* Packetcache
 *
 * Configuration: 
 *
 *
 */

PSHash::PSHash()
{
  myhash=1;
}

PSHash::~PSHash()
{
}

int PSHash::configure(Vector<String> &, ErrorHandler *)
{
    return 0;
}

void PSHash::push(int, Packet *p) 
{
    unsigned short int rzvoff, metaoff;
    const unsigned char *hp;
    const struct pubsub_rzvhdr *rzvp;
    const struct pubsub_metahdr *metap;

    unsigned int hashvalue = 0;

    rzvoff = p->anno_u16(PS_ANNO_RZVHDR);
    metaoff = p->anno_u16(PS_ANNO_METAHDR);
    
    rzvp = reinterpret_cast<const struct pubsub_rzvhdr*>(p->mac_header() + rzvoff);
    metap = reinterpret_cast<const struct pubsub_metahdr*>(p->mac_header() + metaoff);

    /* find VRID */
    if (rzvoff > 0) {
	hp = rzvp->vrid;
    } else if (metap > 0) {
	hp = metap->vrid;
    } else {
	/* no VRID... quit */
	output(0).push(p);
	return;
    }

#if 1
    /* magical hashing algorithm */
    for(unsigned int i=0;i<sizeof(pubsub_id_t);i++) {
	hashvalue += *(hp+i);
    }
#else
    hashvalue = myhash++;
#endif   
    p->set_anno_u32(PS_ANNO_HASHVALUE, hashvalue);
    click_chatter("Packet hashed to value: %d", hashvalue);

    output(0).push(p);
    return;
}

CLICK_ENDDECLS
ELEMENT_REQUIRES(PSCheck)
EXPORT_ELEMENT(PSHash)
