#include <click/config.h>
#include <click/confparse.hh>
#include <click/error.hh>
#include "pscheck.hh"
#include "psload.hh"

CLICK_DECLS

/* Packetcache
 *
 * Configuration: 
 *
 *
 */

PSLoad::PSLoad()
{
    _cache = NULL;
}

PSLoad::~PSLoad()
{
}

int PSLoad::configure(Vector<String> &conf, ErrorHandler *errh)
{
    Element *temp = NULL;

    if (conf.size() < 1) {
	return errh->error("Cache missing");
    }

    temp = cp_element(conf.at(0), this, errh);
    if (!temp) {
	return errh->error("Could not parse PSCache element");
    }

    _cache = (PSCache *)temp->cast("PSCache");
    if (!_cache) {
	return errh->error("Parsed element not PSCache");
    }

//    click_chatter("Parsed PSCache at: %p", (unsigned long)_cache);

    return 0;
}

void PSLoad::push(int, Packet *p) 
{
    unsigned int hashvalue = p->anno_u32(PS_ANNO_HASHVALUE);
    Packet *cachedp;
    cachedp = _cache->load(hashvalue, p);

    if (cachedp != NULL) {
	output(0).push(cachedp);
	p->kill();
    } else {
	click_chatter("Hashvalue: %d, not found", hashvalue);
	output(1).push(p);
    }

    return;
    
}

CLICK_ENDDECLS
ELEMENT_REQUIRES(PSCache)
EXPORT_ELEMENT(PSLoad)
